
import {AxiosRequestConfig, AxiosResponse, CancelTokenStatic} from "axios";

declare interface CommonFunction {
    (): void
}

declare interface BeforeSendFunction {
    (config: AxiosRequestConfig): AxiosRequestConfig
}

declare interface DataFilterFunction {
    (response: AxiosResponse): AxiosResponse
}

declare interface ResponseErrorFunction {
    (error: any, transitive: any[]): any[] | undefined | null
}

declare interface AjaxOption extends AxiosRequestConfig{
    formData?: boolean,
    security?: {
        crypto: boolean
    },
    beforeSend?: BeforeSendFunction[],
    dataFilter?: DataFilterFunction[],
    responseError?: ResponseErrorFunction[],
    success?: DataFilterFunction,
    error?: ResponseErrorFunction,
    complete?: CommonFunction
}

export class Ajax {
    request<T = any, R = AxiosResponse<T>> (config: AxiosRequestConfig): Promise<R>;
    get<T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R>;
    delete<T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R>;
    head<T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R>;
    options<T = any, R = AxiosResponse<T>>(url: string, config?: AxiosRequestConfig): Promise<R>;
    post<T = any, R = AxiosResponse<T>>(url: string, data?: any, config?: AxiosRequestConfig): Promise<R>;
    put<T = any, R = AxiosResponse<T>>(url: string, data?: any, config?: AxiosRequestConfig): Promise<R>;
    patch<T = any, R = AxiosResponse<T>>(url: string, data?: any, config?: AxiosRequestConfig): Promise<R>;
    all<T>(values: (T | Promise<T>)[]): Promise<T[]>;
    getCancelToken(): CancelTokenStatic
    create(config?: AjaxOption): Ajax
}

export default function init(config?: AjaxOption): Ajax;